c ParaDyn - Parallel DYNAMO - molecular dynamics with EAM potentials
c 
c Authored by Steve Plimpton
c   (505) 845-7873, sjplimp@cs.sandia.gov
c   Dept 9221, MS 1111, Sandia National Labs, Albuquerque, NM  87185-1111
c
c Based on the serial DYNAMO code authored by
c   Stephen Foiles (foiles@ca.sandia.gov), Sandia National Labs, Livermore, CA
c   Murray Daw (daw@hubcap.clemson.edu), Clemson University
c
c See the README file for more information
c
      
      subroutine create_atoms
#include "paradyn.h"
#include "mpif.h"
      
      if (node.eq.0) write (6,*) 'Creating Atoms ...'
      
      if (ninterp.eq.0) call error('No interpolation done')
      nvelocities = 0
      
      time_stamp = 0.0D0
      
      perlb(1) = -xlat*latconst/2.0D0
      perlb(2) = -ylat*latconst/2.0D0
      perlb(3) = -zlat*latconst/2.0D0
      perub(1) = -perlb(1)
      perub(2) = -perlb(2)
      perub(3) = -perlb(3)
      
      natoms = 0
      if (lattype.eq.'sc') then
        natoms = xlat*ylat*zlat
      else if (lattype.eq.'bcc') then
        natoms = xlat*ylat*zlat*2
      else if (lattype.eq.'fcc') then
        natoms = xlat*ylat*zlat*4
      else
        call error('Invalid lattice type')
      endif
      
      if (natoms.eq.0) call error('No atoms')
      
      call setup_parallel

c setup RNG

      im = 1
 10   im = im * 2
      if (im.lt.natoms) goto 10
      ia = 365
      ic = 8161
      ivalue = mod(ilatseed,natoms)
      
c create lattices
c  all nodes generate entire lattice, only save the atoms they own
c  generate atom positions sequentially, use simple RNG to 
c   randomize order if ilatseed > 0

      if (lattype.eq.'sc') then
        m = 0
        do k=1,zlat
          do j=1,ylat
            do i=1,xlat
              m = m + 1
              if (ilatseed.eq.0) then
                n = m
              else
 20             ivalue = mod(ia*ivalue+ic,im)
                if (ivalue.ge.natoms) goto 20
                n = ivalue + 1
              endif
              if (n.ge.nstart.and.n.le.nend) then
                ntmp = n - nstart + 1
                tag(ntmp) = m
                itype(ntmp) = 1
                x(1,ntmp) = (i-1)*latconst + perlb(1)
                x(2,ntmp) = (j-1)*latconst + perlb(2)
                x(3,ntmp) = (k-1)*latconst + perlb(3)
              endif
            enddo
          enddo
        enddo
      else if (lattype.eq.'bcc') then
        m = 0
        do k=1,zlat
          do j=1,ylat
            do i=1,xlat
              m = m + 1
              if (ilatseed.eq.0) then
                n = m
              else
 30             ivalue = mod(ia*ivalue+ic,im)
                if (ivalue.ge.natoms) goto 30
                n = ivalue + 1
              endif
              if (n.ge.nstart.and.n.le.nend) then
                ntmp = n - nstart + 1
                tag(ntmp) = m
                itype(ntmp) = 1
                x(1,ntmp) = (i-1)*latconst + perlb(1)
                x(2,ntmp) = (j-1)*latconst + perlb(2)
                x(3,ntmp) = (k-1)*latconst + perlb(3)
              endif
              m = m + 1
              if (ilatseed.eq.0) then
                n = m
              else
 40             ivalue = mod(ia*ivalue+ic,im)
                if (ivalue.ge.natoms) goto 40
                n = ivalue + 1
              endif
              if (n.ge.nstart.and.n.le.nend) then
                ntmp = n - nstart + 1
                tag(ntmp) = m
                itype(ntmp) = 1
                x(1,ntmp) = (i-1)*latconst + perlb(1) + latconst/2.0D0
                x(2,ntmp) = (j-1)*latconst + perlb(2) + latconst/2.0D0
                x(3,ntmp) = (k-1)*latconst + perlb(3) + latconst/2.0D0
              endif
            enddo
          enddo
        enddo
      else if (lattype.eq.'fcc') then
        m = 0
        do k=1,zlat*2
          do j=1,ylat*2
            do i=1,xlat*2
              if (mod(i+j+k,2).eq.1) then
                m = m + 1
                if (ilatseed.eq.0) then
                  n = m
                else
 50               ivalue = mod(ia*ivalue+ic,im)
                  if (ivalue.ge.natoms) goto 50
                  n = ivalue + 1
                endif
                if (n.ge.nstart.and.n.le.nend) then
                  ntmp = n - nstart + 1
                  tag(ntmp) = m
                  itype(ntmp) = 1
                  x(1,ntmp) = (i-1)*latconst/2.0D0 + perlb(1)
                  x(2,ntmp) = (j-1)*latconst/2.0D0 + perlb(2)
                  x(3,ntmp) = (k-1)*latconst/2.0D0 + perlb(3)
                endif
              endif
            enddo
          enddo
        enddo
      endif

      call mpi_barrier(mpi_comm_world,ierror)

c update everone's row and column atom tags and types
c use jneigh and nlarge_lst as temporary storage for counts and displacements

      call mpi_comm_size(row_comm,numcol,ierror)

      call mpi_allgather(nlocal,1,mpi_integer,jneigh,1,mpi_integer,
     $     row_comm,ierror)
      nlarge_lst(1) = 0
      do i = 2,numcol
        nlarge_lst(i) = nlarge_lst(i-1) + jneigh(i-1)
      enddo
      
      call mpi_allgatherv(tag,nlocal,mpi_integer,
     $     tagrow,jneigh,nlarge_lst,mpi_integer,row_comm,ierror)
      call mpi_allgatherv(itype,nlocal,mpi_integer,
     $     ityperow,jneigh,nlarge_lst,mpi_integer,row_comm,ierror)

      call mpi_comm_size(col_comm,numrow,ierror)

      call mpi_allgather(nlocal,1,mpi_integer,jneigh,1,mpi_integer,
     $     col_comm,ierror)
      nlarge_lst(1) = 0
      do i = 2,numrow
        nlarge_lst(i) = nlarge_lst(i-1) + jneigh(i-1)
      enddo

      call mpi_allgatherv(tag,nlocal,mpi_integer,
     $     tagcol,jneigh,nlarge_lst,mpi_integer,col_comm,ierror)
      call mpi_allgatherv(itype,nlocal,mpi_integer,
     $     itypecol,jneigh,nlarge_lst,mpi_integer,col_comm,ierror)
      
c set ntypes and check for consistency with potentials
      
      ntypes = 0
      do i = 1,nlocal
        ntypes = max(ntypes,itype(i))
      enddo
      call mpi_allreduce(ntypes,nmax,1,mpi_integer,mpi_max,
     $     mpi_comm_world,ierror)
      ntypes = nmax
      
      if (ntypes.gt.npots) then
        call error('More atom types than potentials')
      else if (ntypes.lt.npots) then
        if (node.eq.0)
     $       write (6,*) 'WARNING: Less atom types than potentials'
        if (logflag.ne.0.and.node.eq.0)
     $       write (1,*) 'WARNING: Less atom types than potentials'
      endif
      
      call chkper
      
c clear all controls and constraints on new atoms
      
      do i = 1,nlocal
        tmptag(i) = 0
        fixtag(i) = 0
      enddo
      
c zero out velocities

      do i = 1,nlocal
        v(1,i) = 0.0
        v(2,i) = 0.0
        v(3,i) = 0.0
      enddo

c message to screen
      
      if (logflag.ne.0.and.node.eq.0)
     $     write (1,*) 'Atoms Created =',natoms,' Types = ',ntypes
      if (node.eq.0) write (6,*) 'Atoms Created =',natoms,
     $     ' Types = ',ntypes
      
      return
      end
