c ParaDyn - Parallel DYNAMO - molecular dynamics with EAM potentials
c 
c Authored by Steve Plimpton
c   (505) 845-7873, sjplimp@cs.sandia.gov
c   Dept 9221, MS 1111, Sandia National Labs, Albuquerque, NM  87185-1111
c
c Based on the serial DYNAMO code authored by
c   Stephen Foiles (foiles@ca.sandia.gov), Sandia National Labs, Livermore, CA
c   Murray Daw (daw@hubcap.clemson.edu), Clemson University
c
c See the README file for more information
c
      
      subroutine dump_atoms
#include "paradyn.h"
#include "mpif.h"
      integer istatus(mpi_status_size)
 900  format (i7,i3,3f10.4)
      
      if (node.eq.0) write (6,*) 'Dumping Atoms ...'

      if (natoms.eq.0) call error('No atoms defined')

      call mpi_barrier(mpi_comm_world,ierror)
      time1 = mpi_wtime()
      
      if (idumpflag.eq.0.and.node.eq.0) then
        open (unit=2,file=dumpfile)
        close (2,status='delete')
        open (unit=2,file=dumpfile)
      endif
      idumpflag = 1
      
c write to file

      if (node.ne.0) then

        call mpi_recv(ntmp,0,mpi_integer,0,0,
     $       mpi_comm_world,istatus,ierror)
        call mpi_send(tag,nlocal,mpi_integer,0,0,
     $       mpi_comm_world,ierror)
        call mpi_send(itype,nlocal,mpi_integer,0,0,
     $       mpi_comm_world,ierror)
        call mpi_send(x,3*nlocal,mpi_double_precision,0,0,
     $       mpi_comm_world,ierror)

      else
        
        write (2,*) 'ITEM: NUMBER OF ATOMS'
        write (2,*) natoms
        write (2,*) 'ITEM: BOX BOUNDS'
        write (2,*) perlb(1),perub(1)
        write (2,*) perlb(2),perub(2)
        write (2,*) perlb(3),perub(3)
        write (2,*) 'ITEM: TIME'
        write (2,*) time_stamp
        write (2,*) 'ITEM: ATOMS'

        do i = 1,nlocal
          write (2,900) tag(i),itype(i),x(1,i),x(2,i),x(3,i)
        enddo
        
        do iproc = 1,nprocs-1
          call mpi_send(ntmp,0,mpi_integer,iproc,0,
     $         mpi_comm_world,ierror)
          call mpi_recv(tagtmp,maxlocal,mpi_integer,iproc,0,
     $         mpi_comm_world,istatus,ierror)
          call mpi_get_count(istatus,mpi_integer,icnt,ierror)
          call mpi_recv(itypetmp,icnt,mpi_integer,iproc,0,
     $         mpi_comm_world,istatus,ierror)
          call mpi_recv(xtmp,3*icnt,mpi_double_precision,iproc,0,
     $         mpi_comm_world,istatus,ierror)
          do i = 1,icnt
            write (2,900) tagtmp(i),itypetmp(i),
     $           xtmp(1,i),xtmp(2,i),xtmp(3,i)
          enddo
        enddo
        
      endif
      
      call mpi_barrier(mpi_comm_world,ierror)
      time2 = mpi_wtime()
      time_io = time_io + time2-time1
      
      return
      end
