c ParaDyn - Parallel DYNAMO - molecular dynamics with EAM potentials
c 
c Authored by Steve Plimpton
c   (505) 845-7873, sjplimp@cs.sandia.gov
c   Dept 9221, MS 1111, Sandia National Labs, Albuquerque, NM  87185-1111
c
c Based on the serial DYNAMO code authored by
c   Stephen Foiles (foiles@ca.sandia.gov), Sandia National Labs, Livermore, CA
c   Murray Daw (daw@hubcap.clemson.edu), Clemson University
c
c See the README file for more information
c
      
      subroutine chkper
#include "paradyn.h"
      
      permin = 2.0D0*sqrt(cutsq2)
      tmpx = perub(1) - perlb(1)
      tmpy = perub(2) - perlb(2)
      tmpz = perub(3) - perlb(3)
      if (tmpx.le.permin.or.tmpy.le.permin.or.tmpz.le.permin) then
        if (node.eq.0) then
          write (6,*) ' Permin = ',permin
          write (6,*) ' Perlen = ',tmpx,tmpy,tmpz
        endif
        call error('Box length(s) too short')
      endif
      
      return
      end
      
      
      subroutine output
#include "paradyn.h"
 900  format (1x,a,i8,a,e11.4)
 901  format (1x,a,g13.6,a,g13.6,a,g13.6)
 902  format (1x,a,g13.6,a,g13.6,a,g13.6)
 903  format (1x,a,3g13.6)
 904  format (1x,a,i5,a,i8,a,g13.6)
      
      call thermo(ekin,epot,etot,temp,press,volume)
      
      de = abs(etot-eold)
      demax = max(demax,de)
      eold = etot
      
      nave = nave + 1
      avpres = avpres + press
      avtemp = avtemp + temp
      avvol = avvol + volume
      avke = avke + ekin
      avpe = avpe + epot

      do i = 1,6
        avstrs(i) = avstrs(i) + stress(i)
      enddo
      
      if (logflag.ne.0.and.node.eq.0) then
        write (1,*)
        write (1,900) 'TimeStep =',nint(time_stamp/dt),
     $       '  Time =',time_stamp
        write (1,901) ' Total E =',etot,' Pot E =',epot,
     $       ' Kin E =',ekin
        write (1,902) ' Temp =',temp,' Pressure =',press,
     $       ' Volume =',volume
        if (ibdtyp.eq.2) write (1,903) ' Box =',perlen(1),perlen(2),
     $       perlen(3)
        if (ntemps.gt.0) then
          do i = 0,ntemps
            write (1,904) ' Control',i,': Atoms =',tmpnum(i),
     $           ' Temp =',tmpnow(i)
          enddo
        endif
      endif

      if (node.eq.0) then
        write (6,*)
        write (6,900) 'TimeStep =',nint(time_stamp/dt),
     $       '  Time =',time_stamp
        write (6,901) ' Total E =',etot,' Pot E =',epot,
     $       ' Kin E =',ekin
        write (6,902) ' Temp =',temp,' Pressure =',press,
     $       ' Volume =',volume
        if (ibdtyp.eq.2) write (6,903) ' Box =',perlen(1),perlen(2),
     $       perlen(3)
        if (ntemps.gt.0) then
          do i = 0,ntemps
            write (6,904) ' Control',i,': Atoms =',tmpnum(i),
     $           ' Temp =',tmpnow(i)
          enddo
        endif
      endif
      
      return
      end
      
      
      subroutine thermo(ekin,epot,etot,temp,press,volume)
#include "paradyn.h"
#include "mpif.h"
      
      call force(0,0,1)
      
      if (ntemps.gt.0) then
        call update_controls
        call temperature_controls
      endif
      
      ekin = 0.0D0
      do i = 1,nlocal
        vx = v(1,i)
        vy = v(2,i)
        vz = v(3,i)
        ekin = ekin + (vx*vx + vy*vy + vz*vz)*amass(itype(i))
      enddo
      tmp = 0.5D0*ekin
      call mpi_allreduce(tmp,ekin,1,mpi_double_precision,mpi_sum,
     $     mpi_comm_world,ierror)
      temp = 2.0D0/3.0D0/boltz * ekin / natoms
      
      epot = 0.0D0
      do i = 1,nlocal
        epot = epot + e(i)
      enddo
      tmp = epot
      call mpi_allreduce(tmp,epot,1,mpi_double_precision,mpi_sum,
     $     mpi_comm_world,ierror)
      etot = ekin + epot
      
      volume = perlen(1)*perlen(2)*perlen(3)
      
      if (ibdtyp.eq.1) call global_stress

      press = (stress(1)+stress(2)+stress(3))/(3.0D0*volume)
      press = 1.602D6*press
      
      return
      end
      

c accumulate global stress tensor
c if STRESS option chosen, first accumulate from atom-by-atom components
c    computed in force routine
c otherwise just sum across processors

      subroutine global_stress
#include "paradyn.h"
#include "mpif.h"

#ifdef STRESS

c my atoms

      do i = 1,nrow
        stress(1) = stress(1) + strsrow(1,i)
        stress(2) = stress(2) + strsrow(2,i)
        stress(3) = stress(3) + strsrow(3,i)
        stress(4) = stress(4) + strsrow(4,i)
        stress(5) = stress(5) + strsrow(5,i)
        stress(6) = stress(6) + strsrow(6,i)
      enddo
      
c other atoms

      if (newton.eq.2) then
        do i = 1,ncol
          stress(1) = stress(1) + strscol(1,i)
          stress(2) = stress(2) + strscol(2,i)
          stress(3) = stress(3) + strscol(3,i)
          stress(4) = stress(4) + strscol(4,i)
          stress(5) = stress(5) + strscol(5,i)
          stress(6) = stress(6) + strscol(6,i)
        enddo
      endif

      do i = 1,6
        stress(i) = 0.5D0 * stress(i)
      enddo

#else

c if newton = 1, divide by 2 since every component got counted twice

      if (newton.eq.1) then
        do i = 1,6
          stress(i) = 0.5D0 * stress(i)
        enddo
      endif

#endif

c add in kinetic energy term for each atom

      do i = 1,nlocal
        ity = itype(i)
        stress(1) = stress(1) + amass(ity)*v(1,i)*v(1,i)
        stress(2) = stress(2) + amass(ity)*v(2,i)*v(2,i)
        stress(3) = stress(3) + amass(ity)*v(3,i)*v(3,i)
        stress(4) = stress(4) + amass(ity)*v(1,i)*v(2,i)
        stress(5) = stress(5) + amass(ity)*v(1,i)*v(3,i)
        stress(6) = stress(6) + amass(ity)*v(2,i)*v(3,i)
      enddo

c sum each component of global tensor across procs

      call mpi_allreduce(stress,stresstmp,6,mpi_double_precision,
     $     mpi_sum,mpi_comm_world,ierror)
      do i = 1,6
        stress(i) = stresstmp(i)
      enddo
      
      return
      end

      
      subroutine scale
#include "paradyn.h"
      
      if (ibdtyp.eq.1) then
        
        do i = 1,nlocal
          y(1,i) = x(1,i)
          y(2,i) = x(2,i)
          y(3,i) = x(3,i)
          y(4,i) = v(1,i)
          y(5,i) = v(2,i)
          y(6,i) = v(3,i)
        enddo
        
      else
        
        do k = 1,3
          scfact = 1.0D0/perlen(k)
          origin = perlb(k)
          do i = 1,nlocal
            y(k,i) = (x(k,i) - origin)*scfact
            y(k+3,i) = v(k,i)*scfact
          enddo
          y(k,nlocal+1) = perlen(k)
          y(k+3,nlocal+1) = bndvel(k)
        enddo
        
      endif
      
      return
      end
      
      
      subroutine unscale
#include "paradyn.h"
      
      if (ibdtyp.eq.1) then
        
        do i = 1,nlocal
          x(1,i) = y(1,i)
          x(2,i) = y(2,i)
          x(3,i) = y(3,i)
          v(1,i) = y(4,i)
          v(2,i) = y(5,i)
          v(3,i) = y(6,i)
        enddo
        
      else
        
        do k = 1,3
          perlen(k) = y(k,nlocal+1)
          perub(k) = perlb(k) + perlen(k)
          bndvel(k) = y(k+3,nlocal+1)
          scfact = perlen(k)
          origin = perlb(k)
          do i = 1,nlocal
            x(k,i) = y(k,i)*scfact + origin
            v(k,i) = y(3+k,i)*scfact
          enddo
        enddo
        
      endif
      
      return
      end
      
      
      subroutine remap
#include "paradyn.h"
      
      if (ibdtyp.eq.1) then
        
        do i = 1,nlocal
          if (y(1,i).ge.perub(1)) then
            y(1,i) = y(1,i) - perlen(1)
            parity(1,i) = parity(1,i) + 1
          endif
          if (y(1,i).lt.perlb(1)) then
            y(1,i) = y(1,i) + perlen(1)
            parity(1,i) = parity(1,i) - 1
          endif
          if (y(2,i).ge.perub(2)) then
            y(2,i) = y(2,i) - perlen(2)
            parity(2,i) = parity(2,i) + 1
          endif
          if (y(2,i).lt.perlb(2)) then
            y(2,i) = y(2,i) + perlen(2)
            parity(2,i) = parity(2,i) - 1
          endif
          if (y(3,i).ge.perub(3)) then
            y(3,i) = y(3,i) - perlen(3)
            parity(3,i) = parity(3,i) + 1
          endif
          if (y(3,i).lt.perlb(3)) then
            y(3,i) = y(3,i) + perlen(3)
            parity(3,i) = parity(3,i) - 1
          endif
        enddo
        
      else
        
        do i = 1,nlocal
          if (y(1,i).ge.1.0D0) then
            y(1,i) = y(1,i) - 1.0D0
            parity(1,i) = parity(1,i) + 1
          endif
          if (y(1,i).lt.0.0D0) then
            y(1,i) = y(1,i) + 1.0D0
            parity(1,i) = parity(1,i) - 1
          endif
          if (y(2,i).ge.1.0D0) then
            y(2,i) = y(2,i) - 1.0D0
            parity(2,i) = parity(2,i) + 1
          endif
          if (y(2,i).lt.0.0D0) then
            y(2,i) = y(2,i) + 1.0D0
            parity(2,i) = parity(2,i) - 1
          endif
          if (y(3,i).ge.1.0D0) then
            y(3,i) = y(3,i) - 1.0D0
            parity(3,i) = parity(3,i) + 1
          endif
          if (y(3,i).lt.0.0D0) then
            y(3,i) = y(3,i) + 1.0D0
            parity(3,i) = parity(3,i) - 1
          endif
        enddo
        
      endif
      
      return
      end
