c ParaDyn - Parallel DYNAMO - molecular dynamics with EAM potentials
c 
c Authored by Steve Plimpton
c   (505) 845-7873, sjplimp@cs.sandia.gov
c   Dept 9221, MS 1111, Sandia National Labs, Albuquerque, NM  87185-1111
c
c Based on the serial DYNAMO code authored by
c   Stephen Foiles (foiles@ca.sandia.gov), Sandia National Labs, Livermore, CA
c   Murray Daw (daw@hubcap.clemson.edu), Clemson University
c
c See the README file for more information
c
      
      subroutine read_velocities
#include "paradyn.h"
#include "mpif.h"
      integer istatus(mpi_status_size)      
      
      if (node.eq.0) write (6,*) 'Reading Velocities ...'
      
      if (natoms.eq.0) call error('No atoms defined')
      nvelocities = 1
      
c open velocity file - exit if there is a problem

      if (node.eq.0) then
        iflag = -1
        open (unit=9,file=velfile,status='old',err=10)
        iflag = 0
      endif
      
 10   call mpi_bcast(iflag,1,mpi_integer,0,mpi_comm_world,ierror)
      if (iflag.lt.0) call error('Velocity file cannot be opened')
      
c read file

      if (node.eq.0) then
        read (9,*)
        read (9,*) ntmp
        read (9,*)
        read (9,*) bndvel(1),bndvel(2),bndvel(3)
        read (9,*)
        read (9,*)
        read (9,*)
      endif
      
      call mpi_bcast(ntmp,1,mpi_integer,0,mpi_comm_world,ierror)
      if (ntmp.ne.natoms) then
        if (node.eq.0) close (9)
        call error ('Mis-matched velocity file')
      endif

      call mpi_bcast(bndvel,3,mpi_double_precision,0,
     $     mpi_comm_world,ierror)
      
c read each procs set of atom velocities from file and send it
c ncount = # of atoms on processor iproc
c  (using same formula as in setup_parallel)

      if (node.eq.0) then

        do iproc = 0,nprocs-1
          ncount = nint(float(iproc+1)/nprocs*natoms) -
     $         nint(float(iproc)/nprocs*natoms)
          do i = 1,ncount
            read (9,*) tagtmp(i),xtmp(1,i),xtmp(2,i),xtmp(3,i)
          enddo

          if (iproc.eq.0) then
            do i = 1,ncount
              v(1,i) = xtmp(1,i)
              v(2,i) = xtmp(2,i)
              v(3,i) = xtmp(3,i)
            enddo
          else
            call mpi_send(xtmp,3*ncount,mpi_double_precision,
     $           iproc,0,mpi_comm_world,ierror)
          endif

        enddo
        close (9)

      else
        
        call mpi_recv(v,3*maxlocal,
     $       mpi_double_precision,0,0,mpi_comm_world,istatus,ierror)

      endif

      call mpi_barrier(mpi_comm_world,ierror)

      return
      end
