c ParaDyn - Parallel DYNAMO - molecular dynamics with EAM potentials
c 
c Authored by Steve Plimpton
c   (505) 845-7873, sjplimp@cs.sandia.gov
c   Dept 1421, MS 1111, Sandia National Labs, Albuquerque, NM  87185-1111
c
c Based on the serial DYNAMO code authored by
c   Stephen Foiles (foiles@ca.sandia.gov), Sandia National Labs, Livermore, CA
c   Murray Daw (daw@hubcap.clemson.edu), Clemson University
c
c Last modified:  20 April 1995
c
c See the README file for more information
c
      
c Randomizes order of atoms for force-decomposition with user seed
c both in/out are "read atoms" and "dump atoms" format files
c see OVERVIEW file for format
c only randomizes 1st snapshot of atom file

c syntax:  randomize iseed in.file out.file velocity_in.file velocity_out.file

      program randomize
      parameter (maxatom=20000)
      real*8 perlb(3),perub(3),x(3,maxatom),random
      integer tag(maxatom),itype(maxatom),order(maxatom)
      character*80 infile,outfile,vel_infile,vel_outfile
      character*10 arg
      integer velflag
 900  format (i5,i3,3f10.4)
      
c read inputs

      if (iargc().eq.3) then
        call getarg(1,arg)
        read (arg,'(i)') iseed
        call getarg(2,infile)
        call getarg(3,outfile)
      else if (iargc().eq.5) then
        call getarg(1,arg)
        read (arg,'(i)') iseed
        call getarg(2,infile)
        call getarg(3,outfile)
        call getarg(4,vel_infile)
        call getarg(5,vel_outfile)
      else
        write (6,*) 'Random # seed (8 digits or less)'
        read (5,*) iseed
        write (6,*) 'Input file'
        read (5,*) infile
        write (6,*) 'Output file'
        read (5,*) outfile
        write (6,*) 'Also re-order a velocity file (0) no (1) yes'
        read (5,*) velflag
        if (velflag.eq.1) then
          write (6,*) 'Velocity input file'
          read (5,*) vel_infile
          write (6,*) 'Velocity output file'
          read (5,*) vel_outfile
        endif
      endif

c read atom file header

      open (1,file=infile,status='old')
      
      read (1,*)
      read (1,*) natoms
      read (1,*)
      read (1,*) perlb(1),perub(1)
      read (1,*) perlb(2),perub(2)
      read (1,*) perlb(3),perub(3)
      read (1,*)
      read (1,*) time_stamp
      read (1,*)
      
      if (natoms.gt.maxatom) then
        write (6,*) 'Too many atoms'
        stop
      endif

c read velocity file header

      if (velflag.eq.1) then

        open (2,file=vel_infile,status='old')
        
        read (2,*)
        read (2,*) natoms
        read (2,*)
        read (2,*) perlb(1),perub(1)
        read (2,*) perlb(2),perub(2)
        read (2,*) perlb(3),perub(3)
        read (2,*)
        read (2,*) time_stamp
        read (2,*)
        
        if (natoms.gt.maxatom) then
          write (6,*) 'Too many atoms'
          stop
        endif

      endif
      
c read atoms

      do i = 1,natoms
        order(i) = i
        read (1,*) tag(i),itype(i),x(1,i),x(2,i),x(3,i)
      enddo
      
      close (1)

c read velocities

      if (velflag.eq.1) then

        do i = 1,natoms
          read (2,*) tag(i),itype(i),v(1,i),v(2,i),v(3,i)
        enddo
        
        close (2)

      endif

c randomize order via order array

      do i = 1,natoms
        j = natoms*random(iseed) + 1
        if (j.gt.natoms) j = natoms
        k = order(i)
        order(i) = order(j)
        order(j) = k
      enddo
      
c write header of new atom output file

      open (1,file=outfile)
      close (1,status='delete')
      open (1,file=outfile)

      write (1,*) 'ITEM: NUMBER OF ATOMS'
      write (1,*) natoms
      write (1,*) 'ITEM: BOX BOUNDS'
      write (1,*) perlb(1),perub(1)
      write (1,*) perlb(2),perub(2)
      write (1,*) perlb(3),perub(3)
      write (1,*) 'ITEM: TIME'
      write (1,*) time_stamp
      write (1,*) 'ITEM: ATOMS'

c write header of new velocity output file

      if (velflag.eq.1) then

        open (2,file=vel_outfile)
        close (2,status='delete')
        open (2,file=vel_outfile)
        
        write (2,*) 'ITEM: NUMBER OF ATOMS'
        write (2,*) natoms
        write (2,*) 'ITEM: BOX BOUNDS'
        write (2,*) perlb(1),perub(1)
        write (2,*) perlb(2),perub(2)
        write (2,*) perlb(3),perub(3)
        write (2,*) 'ITEM: TIME'
        write (2,*) time_stamp
        write (2,*) 'ITEM: ATOMS'
      
      endif

c write out atoms in new ordering

      do i = 1,natoms
        j = order(i)
        write (1,900) tag(j),itype(j),x(1,j),x(2,j),x(3,j)
      enddo
      
      close (1)

c write out velocities in new ordering

      if (velflag.eq.1) then

        do i = 1,natoms
          j = order(i)
          write (2,900) tag(j),itype(j),v(1,j),v(2,j),v(3,j)
        enddo
        
        close (2)
        
      endif

      end


c ***************
c Subroutine
c ***************

      real*8 function random(iseed)
      implicit real*8 (a-h,o-z)
      parameter (a=16807.0D0,rm=2147483647.0D0)
      
      seed = iseed
      seed = mod(a*seed,rm)
      random = seed/rm
      iseed = seed

      return
      end

